;********************************************************************
;
; Author        : ADI - Apps            www.analog.com/MicroConverter
;
; Date          : 15 October 1999
;
; File          : 812cal00.asm  ******** PRE-RELEASE VERSION ********
;                                        ===================
; Hardware      : ADuC812
;
; Include Files : 812cal.asm - ADC calibration routines
;                 UARTIO.asm - serial I/O routines
;
; Description   : test program to illustrate function of calibration
;                 routines contained in the '812cal.asm' file.  uses
;                 the UART to communicate with a PC or dumb-terminal
;                 via RS232 at 9600baud (assuming 11.0592MHz clock).
;                 specific terminal configuration is 9600/8/n/1.
;                 simply download code to ADuC812 eval board, connect
;                 eval board serial connector to PC or dumb-terminal,
;                 and reset board.  simple user interface is provided
;                 on the terminal window.
;
; NOTE          : this is a PRE-RELEASE version.  when released, this
;                 code will be available on the MicroConverter web
;                 site and will be complemented by a technical note
;                 describing the calibration routines in detail.
;
;********************************************************************

$MOD812                 ; use ADuC812&8052 predefined symbols

LED        EQU  P3.4    ; P3.4 is red LED on eval board

;____________________________________________________________________
                                               ; VARIABLE DEFINITIONS

DSEG AT 30h

FADCOFSL:  DS  1        ; factory default offset coefficients
FADCOFSH:  DS  1

FADCGAINL: DS  1        ; factory default gain coefficients
FADCGAINH: DS  1

;____________________________________________________________________
                                                       ; MAIN PROGRAM
CSEG AT 0000h

        MOV     SP,#127

; STORE FACTORY DEFAULT CALIBRATION COEFFICIENTS....

        MOV     FADCOFSL,ADCOFSL   ; offset coefficient low byte..
        MOV     FADCOFSH,ADCOFSH   ; ..and high byte
        MOV     FADCGAINL,ADCGAINL ; gain coefficient low byte..
        MOV     FADCGAINH,ADCGAINH ; ..and high byte

; CONFIGURE UART, DACs, AND ADC....

        MOV     SCON,#52h       ; config UART for 9600baud..
        MOV     TMOD,#20h       ; ..assuming 11.0592MHz Mclk
        MOV     TH1,#-3
        SETB    TR1

        MOV     DACCON,#05Fh    ; DAC0=Vref, DAC1=VDD
        MOV     DAC0H,#00Fh
        MOV     DAC1H,#00Fh
        MOV     DAC0L,#0FFh
        MOV     DAC1L,#0FFh

        MOV     ADCCON1,#060h   ; ADC on, normal mode
        MOV     ADCCON2,#0
        CALL    DELAY65ms       ; wait for Vref to turn on & settle

; BEGIN MAIN CODE LOOP....

MAIN:   SETB    LED             ; turn LED on
        MOV     DPTR,#TITLE     ; display title text
        CALL    SENDSTRING

; DISPLAY VALUES OF CALIBRATION COEFFICIENTS....

        MOV     DPTR,#ADCOFSMSG   ; display ADCOFS registers:
        CALL    SENDSTRING
        MOV     A,FADCOFSH        ; factory defaults..
        CALL    SENDVAL
        MOV     A,FADCOFSL
        CALL    SENDVAL
        MOV     DPTR,#PRESENTMSG  ; ..and current values
        CALL    SENDSTRING
        MOV     A,ADCOFSH
        CALL    SENDVAL
        MOV     A,ADCOFSL
        CALL    SENDVAL

        MOV     DPTR,#ADCGAINMSG  ; display ADCGAIN registers:
        CALL    SENDSTRING
        MOV     A,FADCGAINH       ; factory defaults..
        CALL    SENDVAL
        MOV     A,FADCGAINL
        CALL    SENDVAL
        MOV     DPTR,#PRESENTMSG  ; ..and current values
        CALL    SENDSTRING
        MOV     A,ADCGAINH
        CALL    SENDVAL
        MOV     A,ADCGAINL
        CALL    SENDVAL

; PERFORM 10 ADC CONVERSIONS AND DISPLAY RESULTS....

        CLR     ADCI
        SETB    SCONV           ; perform ADC conversion..
        JNB     ADCI,$          ; ..but ignore first result
        MOV     DPTR,#ADCMSG    ; send text message identifying..
        CALL    SENDSTRING      ; ..ADC conversion step
        MOV     A,ADCCON2
        CALL    HEX2ASCII       ; indicate channel number
        CALL    SENDCHAR
        MOV     DPTR,#COLONMSG  ; end line with ': '
        CALL    SENDSTRING
        MOV     B,#10           ; B = number of conversions to take
CONV1:  CLR     ADCI
        SETB    SCONV           ; perform ADC conversion..
        JNB     ADCI,$
        MOV     A,ADCDATAH      ; ..and display result
        CALL    HEX2ASCII       ;         (with channel ID removed)
        CALL    SENDCHAR
        MOV     A,ADCDATAL
        CALL    SENDVAL
        DEC     B
        MOV     A,B
        JZ      MENU            ; after 10 conversions, exit loop
        MOV     A,#','
        CALL    SENDCHAR        ; send comma between data points
        JMP     CONV1

; DISPLAY MAIN MENU AND GET USER OPTION INPUT....

MENU:   MOV     DPTR,#MENUMSG   ; display main options menu
        CALL    SENDSTRING
        CLR     LED             ; turn LED off
GETOPT: CALL    GETCHAR         ; get selection char (from user)
        MOV     B,A             ; store typed character
        CLR     C
        SUBB    A,#'0'
        CJNE    A,#4,$+3        ; see if char is a valid option:
        JNC     GETOPT          ; if not valid, try again
        XCH     A,B             ; otherwise, if valid..
        CALL    SENDCHAR        ; ..then echo received character
        XCH     A,B
        MOV     B,#3
        MUL     AB
        MOV     DPTR,#JMPTBL    ; jump to 1 of the 3 options below..
        JMP     @A+DPTR         ; ..if user has entered 0, 1, or 2

JMPTBL: LJMP    MAIN            ; 0 = convert again & refresh menu
        LJMP    CALZS           ; 1 = perform zero-scale calibration
        LJMP    CALFS           ; 2 = perform full-scale calibration
        LJMP    NEWCH           ; 3 = select new primary ADC channel

; IF USER PRESSED '1', PERFORM ZERO-SCALE CALIBRATION....

CALZS:  MOV     DPTR,#APPLY0MSG ; prompt user to apply 0V input..
        CALL    SENDSTRING      ; ..and to enter input channel
GETCH1: CALL    GETCHAR         ; get channel number from user
        CJNE    A,#27,NESC01    ; if ESC character received..
        JMP     MAIN            ; ..then jump back to main loop
NESC01: MOV     B,A             ; store typed character
        CALL    ASCII2HEX
        CJNE    A,#8,$+3        ; if channel# is greater than 7..
        JNC     GETCH1          ; ..then try again
        XCH     A,B             ; if channel# is valid (<=7)..
        CALL    SENDCHAR        ; ..then echo received character
        XCH     A,B

; note: channel number is passed to calibration routine via ACC

        SETB    LED             ; turn LED on
        MOV     DPTR,#CALZSMSG  ; indicate "calibrating..."
        CALL    SENDSTRING
        CALL    CALZEROSCALE    ; perform zero-scale calibration
        JC      FAIL            ; if failure, indicate "FAIL" to user
        MOV     DPTR,#PASSMSG   ; otherwise, indicate "OK"
        CALL    SENDSTRING
        CALL    DELAY65ms
        JMP     MAIN

; IF USER PRESSED '2', PERFORM FULL-SCALE CALIBRATION....

CALFS:  MOV     DPTR,#APPLY1MSG ; prompt user to apply Vref input..
        CALL    SENDSTRING      ; ..and to enter input channel
GETCH2: CALL    GETCHAR         ; get channel number from user
        CJNE    A,#27,NESC02    ; if ESC character received..
        JMP     MAIN            ; ..then jump back to main loop
NESC02: MOV     B,A             ; store typed character
        CALL    ASCII2HEX
        CJNE    A,#8,$+3        ; if channel# is greater than 7..
        JNC     GETCH2          ; ..then try again
        XCH     A,B             ; if channel# is valid (<=7)..
        CALL    SENDCHAR        ; ..then echo received character
        XCH     A,B

; note: channel number is passed to calibration routine via ACC

        SETB    LED             ; turn LED on
        MOV     DPTR,#CALFSMSG  ; indicate "calibrating..."
        CALL    SENDSTRING
        CALL    CALFULLSCALE    ; perform full-scale calibration
        JC      FAIL            ; if failure, indicate "FAIL" to user
        MOV     DPTR,#PASSMSG   ; otherwise, indicate "OK"
        CALL    SENDSTRING
        CALL    DELAY65ms
        JMP     MAIN

; IF EITHER CALIBRATION FAILS, DISPLAY "FAIL" MESSAGE....

FAIL:   MOV     DPTR,#FAILMSG
        CALL    SENDSTRING
        CALL    DELAY65ms
        JMP     MAIN

; IF USER PRESSED '3', GET NEW PRIMARY ADC CHANNEL FROM USER....

NEWCH:  MOV     DPTR,#NEWCHMSG  ; ask user to enter new channel#
        CALL    SENDSTRING
GETCH3: CALL    GETCHAR         ; get channel number from user
        CJNE    A,#27,NESC03    ; if ESC character received..
        JMP     MAIN            ; ..then jump back to main loop
NESC03: MOV     B,A             ; store typed character
        CALL    ASCII2HEX
        CJNE    A,#9,$+3        ; if channel# is greater than 7..
        JNC     GETCH3          ; ..then try again
        MOV     ADCCON2,A       ; otherwise, change channel to ACC..
        XCH     A,B
        CALL    SENDCHAR        ; ..and echo received character
        JMP     MAIN

;____________________________________________________________________
                                           ; SUBROUTINE INCLUDE FILES

$INCLUDE(812CAL.asm)
$INCLUDE(UARTIO.asm)

;____________________________________________________________________
                                                   ; TEXT DATA TABLES

TITLE:       DB 10,13,'_____________________________________________'
             DB '___________________',10,10,13
             DB 'Analog Devices MicroConverter ADuC812 - '
             DB 'Calibration Demo Routine',10,13,0
ADCMSG:      DB 10,10,13,'ADC results (channel ',0
COLONMSG:    DB '): ',0
ADCOFSMSG:   DB 10,13,'offset coefficients (ADCOFS):  default=',0
ADCGAINMSG:  DB 10,13,'gain coefficients  (ADCGAIN):  default=',0
PRESENTMSG:  DB ' / present=',0
MENUMSG:     DB 10,10,13
             DB 'options:  0 - convert again & refresh screen',10,13
             DB '          1 - perform zero-scale calibration',10,13
             DB '          2 - perform full-scale calibration',10,13
             DB '          3 - select new primary ADC channel'
             DB '  ...choose option:',0
APPLY0MSG:   DB 10,10,13
  DB 'apply 0V to an ADC input, and enter the channel number (0-7)'
  DB 10,13
  DB 'of that input to launch the zero-scale calibration routine:',0
APPLY1MSG:   DB 10,10,13
  DB 'apply Vref to an ADC input, and enter the channel number (0-7)'
  DB 10,13
  DB 'of that input to launch the full-scale calibration routine:',0
CALZSMSG:    DB 10,10,13,'performing zero-scale calibration....',0
CALFSMSG:    DB 10,10,13,'performing full-scale calibration....',0
PASSMSG:     DB 'OK',0
FAILMSG:     DB 'FAIL!',0
NEWCHMSG:    DB 10,10,13
             DB 'enter new channel for ADC conversions (0-8):',0

;____________________________________________________________________

END

